from time import sleep


def auto_retry_network_connections(func):
    def wrapper(*args, **kwargs):
        MAX_TIMES = 5
        for time in range(1, MAX_TIMES + 1):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                print(f"{time}th Network connection failed with an exception: {e}")
                if time < MAX_TIMES:
                    print("Retrying...")
                    sleep(1)
                else:
                    print("Reach max retries. Stop.")
                    raise e
    return wrapper



from bs4 import BeautifulSoup
import requests

from buptmw.constants import Session, CAS as CASE
from buptmw.utlis.auto_retry import auto_retry_network_connections


class CAS:
    def __init__(self, username, password):
        self.username = username
        self.password = password
        self._history = []
        try:
            self._login()
        except requests.HTTPError as e:
            if e.response.status_code not in [401, 423]:
                raise e
            self.status = False
        else:
            self.status = True

    def _get_session(self):
        session = requests.Session()
        session.headers["User-Agent"] = Session.USER_AGENT
        return session

    @auto_retry_network_connections
    def _login(self):
        self.session = self._get_session()
        resp = self.session.get(url=CASE.LOGIN)
        varid = BeautifulSoup(
            resp.text, "lxml"
        ).find(attrs={"name": "execution"})["value"]
        post_data = {
            "username": self.username,
            "password": self.password,
            "type": "username_password",
            "submit": "LOGIN",
            "_eventId": "submit",
            "execution": varid
        }
        resp = self.session.post(url=CASE.LOGIN, data=post_data)
        resp.raise_for_status()

    # redirect all undefined methods to self.session
    def __getattr__(self, name):
        attr = getattr(self.session, name)
        if callable(attr):
            return attr.__get__(self.session, type(self.session))
        else:
            return attr



from typing import Type

from .bupt import BUPT_Auth
from .plates.cas import CAS
from .plates.uc import UC
from .plates.ucloud import Ucloud

Type_BUPT_Auth = Type[BUPT_Auth]
Type_CAS = Type[CAS]
Type_UC = Type[UC]
Type_Ucloud = Type[Ucloud]

__all__ = [
    "Type_BUPT_Auth",
    "Type_CAS",
    "Type_UC",
    "Type_Ucloud"
]


from typing import Type, Optional
from time import time

from buptmw.constants import UC as UCE
from .cas import CAS


class UC:
    class Exceptions:
        class NeedCAS(Exception):
            def __init__(self):
                self.message = "Need a BUPT.CAS to init."
                super().__init__(self.message)

    def __init__(self, cas: Optional[Type[CAS]] = None):
        if cas is None or not isinstance(cas, CAS):
            raise self.Exceptions.NeedCAS()
        self.cas = cas
        self._login()

        # get personal info and store it in self.
        resp = self.get(
            UCE.CHECK,
            params={"selfTimestamp": int(time() * 1000)}
        )
        data = resp.json()["data"]
        self.name = data["name"]
        self.buptid = data["schoolid"]

    def _login(self):
        self.cas.get(UCE.LOGIN)

    def check(self):
        resp = self.get(
            UCE.CHECK,
            params={"selfTimestamp": int(time() * 1000)}
        )
        code = resp.json()["code"]
        if code == 0:
            return True
        return False

    # redirect all undefined methods to self.cas
    def __getattr__(self, name):
        attr = getattr(self.cas, name)
        if callable(attr):
            return attr.__get__(self.cas, type(self.cas))
        else:
            return attr
